%   HSRIVC   Computes the Hammerstein continuous-time  model parameters from regular sampled I/O data
%	by the Hammerstein Simplified Refined Instrumental Variable HSRIVC for  CT Output-Error model,
%
%       y(t) = [B(s)/F(s)]ubar(t) + e(t)
%       ubar(t) = u1(t) + alpha1 u2(t) + ... + alphan un+1(t)
%       u1,u2...un(t) represent basics functions generated by the users.
%------------------------------------------------------------------------------------
%	Usage :
%
%       [Modlin, Infos] = hsrivc(Z, nn, lambda0, t0, init_method, tol, maxiter, arma_method);
%------------------------------------------------------------------------------------
%   Modlin : returns the estimated linearized model in an IDPOLY object format
%       along with structure information.
%       y(t) = [B(s)/F(s)]u1(t) + alpha1[B(s)/F(s)]u2(t) +... + alphan[B(s)/F(s)]un+1(t) + e(t)
%       For the exact format of M see also help IDPOLY.
%
%
%   Infos is a structure containing the following fields
%   modelLin   : the linear part continuous-time model
%   NLinCoeff  : coefficients of the nonlinear function
%   Noisemodel : the discrete-time noise model
%   InfoCriterion : Information criterion
%           RT2       : coefficient of determination
%           YIC       : Young's Information Criterion
%           AIC       : Akaike Information criterion
%   Cond_IPM   : condition number of the Instrumental Product Matrix
%   iternum    : number of iteration for convergence
%   present    : presentation of the model under hammerstein form
%------------------------------------------------------------------------------------
%   Z : The estimation data in IDDATA object format.
%       Data have to be equally sampled (see srivc for nonequally sampled data)
%       See help IDDATA
% 	nn = [nb nf nk] where
% 		 	 nb : number of parameters to be estimated for the numerator of the CT TF plant model
%            nf : number of parameters to be estimated for the denominator of the CT TF plant model
% 		  	 nk : delay of the model (integer number of sampling period Ts)
%
%	Optional Parameters :
%   lambda0 : [lambda(1) ... lambda(nu)]: cut-off frequency of the Poisson filter chain element
%             required for the initial parameter estimation by the ivgpmf routine
%             lambda(i) is the design parameter associated to the transfer function
%             of the ith input.
%   t0      : for regularly sampled data, t0=n0 is the starting estimation index (n0=t0/Ts)
%	          By default, it is set on to 1.
%   init_method : method used to initialize the srivc routine
%                 'sriv'  : discrete-time version of the SRIVC method
%                           'sriv' will be used by default in the case of equally sampled data
%                           The discrete-time model is converted into continuous-time
%                           does not require to specify lambda0.
%                 'ivgpmf': IV-based GPMF estimation method.
%                           the cut-off frequency (lambda0) can be then specified
%                           if it is not specified, then lambda0 is automatically set
%                           to ws/4 where ws is the sampling frequency in rad/sec
%                           'ivgpmf' will be used by default
%                               if lambda0 is given as third input argument,
%                 'lssvf' : LS-based SVF estimation method
%                           requires to specify lambda0
%   tol     : tolerance value (in %) on relative error of the estimated parameters,
%  	          tol is used to stop the iterative search (default value: 1e-2 %)
%   maxiter : maximum of loops for the iterations (default value: 50)
%   arma_method : method used to initialize the srivc routine
%                 'pem'   : use of the PEM SID toolbox method (default)
%                 'ivarma': use of the IVARMA CONTSID toolbox method (see
%                 help ivarma)
%------------------------------------------------------------------------------------
%	See for further explanations :
%
%	V. Laurain, M. Gilson, H. Garnier, and P. C. Young. "Refined instrumental
%    variable methods for identification of Hammerstein continuous
%    time Box-Jenkins models". In proceedings of the 47th IEEE Conference
%    on Decision and Control, Cancun, Mexico, Dec 2008.
%------------------------------------------------------------------------------------
%	authors  : Laurain Vincent
%	date     : 20 March 2007
%	revision : 20 June 2009 (CONTSID integration)
%   name     : hsrivc.m
%   initial name: nlsrivc.m renamed by HG 10/05/2013
%
%	CRAN - Centre de Recherche en Automatique de Nancy
%	e-mail : hugues.garnier@univ-lorraine.fr


